package cat.xaviermarin.pm.domain;

import java.io.Serializable;
import java.util.Collection;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

@Entity
@Table(name="PROJECTS") 
public class Project implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
	
	private String name;
	private String description;
	private String webAddress;
	@ManyToOne
    @JoinColumn(name = "director_id")
	private User director;
	@ManyToOne
    @JoinColumn(name = "chief_id")
	private User chief;
	
	@OneToMany(mappedBy="project")
	private Collection<Milestone> milestones;
	
	@OneToMany(mappedBy="project")
	private Collection<Task> tasks;
	
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getWebAddress() {
		return webAddress;
	}

	public void setWebAddress(String webAddress) {
		this.webAddress = webAddress;
	}

	public User getDirector() {
		return director;
	}

	public void setDirector(User director) {
		this.director = director;
	}

	public User getChief() {
		return chief;
	}

	public void setChief(User chief) {
		this.chief = chief;
	}		
	
	public Collection<Milestone> getMilestones() {
		return milestones;
	}

	public void setMilestones(Collection<Milestone> milestones) {
		this.milestones = milestones;
	}
	
	public Collection<Task> getTasks() {
		return tasks;
	}

	public void setTasks(Collection<Task> tasks) {
		this.tasks = tasks;
	}

	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("Id: " + id + "; ");
		buffer.append("Name: " + name + "; ");
		buffer.append("Description: " + description + "; ");
		buffer.append("WebAddress: " + webAddress + "; ");
		buffer.append("Director: " + (director==null?"":director.getId()) + "; ");
		buffer.append("Chief: " + (chief==null?"":chief.getId()));
		return buffer.toString();
	}

	/* 
	 * BUG: http://stackoverflow.com/questions/3181339/right-way-to-implement-equals-contract/3181374#3181374
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chief == null) ? 0 : chief.hashCode());
		result = prime * result
				+ ((description == null) ? 0 : description.hashCode());
		result = prime * result
				+ ((director == null) ? 0 : director.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result
				+ ((webAddress == null) ? 0 : webAddress.hashCode());
		return result;
	}

	/* 
	 * BUG: http://stackoverflow.com/questions/3181339/right-way-to-implement-equals-contract/3181374#3181374
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Project other = (Project) obj;
		if (chief == null) {
			if (other.chief != null) {
				return false;
			}
		} else if (!chief.equals(other.chief)) {
			return false;
		}
		if (description == null) {
			if (other.description != null) {
				return false;
			}
		} else if (!description.equals(other.description)) {
			return false;
		}
		if (director == null) {
			if (other.director != null) {
				return false;
			}
		} else if (!director.equals(other.director)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (webAddress == null) {
			if (other.webAddress != null) {
				return false;
			}
		} else if (!webAddress.equals(other.webAddress)) {
			return false;
		}
		return true;
	}
	
	
}
