package cat.xaviermarin.pm.service.jpa;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cat.xaviermarin.pm.domain.Milestone;
import cat.xaviermarin.pm.repository.MilestoneRepository;
import cat.xaviermarin.pm.service.MilestoneService;

import com.google.common.collect.Lists;

@Service("milestoneService")
@Repository
@Transactional
public class MilestoneServiceImpl implements MilestoneService {

	final static Logger logger = LoggerFactory.getLogger(MilestoneServiceImpl.class);
	
	@Autowired
	private MilestoneRepository milestoneRepository;
	
	@Transactional(readOnly=true)
	public List<Milestone> findAll() {
		logger.info("Finding all milestones");
		return Lists.newArrayList(milestoneRepository.findAll());
	}

	@Transactional(readOnly=true)
	public List<Milestone> findAllByProjectId(Long id) {
		logger.info("Finding all milestones by projectId: {}", id);
		return Lists.newArrayList(milestoneRepository.findAllByProjectId(id));
	}
	
	@Transactional(readOnly=true)
	public Milestone findById(Long id) {
		return milestoneRepository.findOne(id);
	}

	public Milestone save(Milestone milestone) {
		return milestoneRepository.save(milestone);
	}
	
	public void delete(Long id){
		milestoneRepository.delete(id);
	}
	
}
