package cat.xaviermarin.pm.service.jpa;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cat.xaviermarin.pm.domain.Task;
import cat.xaviermarin.pm.repository.TaskRepository;
import cat.xaviermarin.pm.service.TaskService;

import com.google.common.collect.Lists;

@Service("taskService")
@Repository
@Transactional
public class TaskServiceImpl implements TaskService {

	final static Logger logger = LoggerFactory.getLogger(TaskServiceImpl.class);
	
	@Autowired
	private TaskRepository taskRepository;
	
	@Transactional(readOnly=true)
	public List<Task> findAll() {
		logger.info("Finding all tasks");
		return Lists.newArrayList(taskRepository.findAll());
	}

	@Transactional(readOnly=true)
	public List<Task> findAllByProjectId(Long id) {
		logger.info("Finding all tasks by projectId: {}", id);
		return Lists.newArrayList(taskRepository.findAllByProjectId(id));
	}
	
	@Transactional(readOnly=true)
	public Task findById(Long id) {
		return taskRepository.findOne(id);
	}

	public Task save(Task task) {
		return taskRepository.save(task);
	}
	
	public void delete(Long id){
		taskRepository.delete(id);
	}
	
}
