package cat.xaviermarin.pm.web.controller;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.binding.message.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.webflow.execution.RequestContext;

import cat.xaviermarin.pm.domain.Assignation;
import cat.xaviermarin.pm.service.AssignationService;
import cat.xaviermarin.pm.web.view.AssignationBean;
import cat.xaviermarin.pm.web.view.AssignationListBean;

@Component("assignationController")
public class AssignationController extends BaseController {

	private static final Logger logger = LoggerFactory.getLogger(AssignationController.class);

	@Autowired
	private AssignationService assignationService;
	
	//
	// BEANS MANAGEMENT
	//
	
	// RECOVERS MILESTONES LIST AND SET IT IN LISTBEAN
	public AssignationListBean getAssignationListBean() {		
		logger.info("AssignationController.getAssignationListBean");
		AssignationListBean assignationListBean = new AssignationListBean();		
		assignationListBean.setAssignations(assignationService.findAll());		
		return assignationListBean;		
	}
	
	// CREATES A NEW EMPTY MILESTONE BEAN
	public AssignationBean newAssignationBean() {		
		logger.info("newAssignationBean");		
		
		AssignationBean assignationBean = new AssignationBean();		
		Assignation assignation = new Assignation();
		assignationBean.setAssignation(assignation);			       
		
        assignationBean.setAssignations(this.getAssignations());		
		return assignationBean;		
	}

	// CREATES A FULL MILESTONE BEAN
	public AssignationBean getAssignationBean(RequestContext context) {		
		logger.info("getAssignationBean");
		
		AssignationBean assignationBean = new AssignationBean();	
		Assignation assignation = new Assignation();		
		assignation = this.getAssignation(context);	
		assignationBean.setAssignation(assignation);	
		
		assignationBean.setAssignations(this.getAssignations());
		
		return assignationBean;		
	}
	
	//
	// VIEW ACTIONS
	//
	
	public void deleteAssignation(RequestContext context) {
		logger.info("deleteAssignation");
		Long id = context.getRequestScope().getLong("assignationId");
		logger.info("Selected assignation id: {}", id);
		assignationService.delete(id);
	}
	
	public Assignation showAssignation(RequestContext context) {		
		logger.info("showAssignation");		
		return this.getAssignation(context);
	}	
	
	public void saveAssignation(RequestContext context) {	
		logger.info("saveAssignation");		
		// Retrieve contact bean
		AssignationBean assignationBean = (AssignationBean) context.getFlowScope().get("assignationBean");
		Assignation assignation = assignationBean.getAssignation();		
		assignationService.save(assignation);				
		logger.info("Assignation {} {} saved successfully", assignation.getId(), assignation.getDescription());
		//logger.info("<<<<<<<< assignation.getAssignation().getName(): {}", assignation.getAssignation().getName());
		context.getMessageContext().addMessage(new MessageBuilder().info().code("message_assignation_save_success").build());		
	}	

	//
	// PRIVATE
	//
	
	private Assignation getAssignation(RequestContext context) {
		logger.info("getAssignation");		
		Long id = context.getRequestScope().getLong("assignationId");		
		logger.info("Selected assignation id: {}", id);		
		return assignationService.findById(id);			
	}
	
	private List<Assignation> getAssignations() {
		logger.info("getAssignations");		
		List<Assignation> assignations = this.assignationService.findAll();         
        return assignations;
	}
}
