package cat.xaviermarin.pm.web.controller;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.binding.message.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.webflow.execution.RequestContext;

import cat.xaviermarin.pm.domain.Project;
import cat.xaviermarin.pm.domain.User;
import cat.xaviermarin.pm.service.ProjectService;
import cat.xaviermarin.pm.service.UserService;
import cat.xaviermarin.pm.web.view.ProjectBean;
import cat.xaviermarin.pm.web.view.ProjectListBean;

@Component("dashboardController")
public class DashboardController extends BaseController {

	private static final Logger logger = LoggerFactory.getLogger(DashboardController.class);

	@Autowired
	private ProjectService projectService;
	
	@Autowired
	private UserService userService;
	
	//
	// BEANS MANAGEMENT
	//
	
	// RECOVERS PROJECTS LIST AND SET IT IN LISTBEAN
	public ProjectListBean getProjectListBean() {		
		logger.info("getProjectListBean");
		ProjectListBean projectListBean = new ProjectListBean();		
		projectListBean.setProjects(projectService.findAll());		
		return projectListBean;		
	}
	
	// CREATES A NEW EMPTY PROJECT BEAN
	public ProjectBean newProjectBean() {		
		logger.info("newProjectBean");		
		
		ProjectBean projectBean = new ProjectBean();		
		Project project = new Project();
		projectBean.setProject(project);			       
		
        projectBean.setUsers(this.getUsers());		
		return projectBean;		
	}

	// CREATES A FULL PROJECT BEAN
	public ProjectBean getProjectBean(RequestContext context) {		
		logger.info("getProjectBean");
		
		ProjectBean projectBean = new ProjectBean();	
		Project project = new Project();		
		project = this.getProject(context);	
		projectBean.setProject(project);	
		
		projectBean.setUsers(this.getUsers());
		
		return projectBean;		
	}
	
	//
	// VIEW ACTIONS
	//
	
	public void deleteProject(RequestContext context) {
		logger.info("deleteProject");
		Long id = context.getRequestScope().getLong("projectId");
		logger.info("Selected project id: {}", id);
		projectService.delete(id);
	}
	
	public Project showProject(RequestContext context) {		
		logger.info("showProject");		
		return this.getProject(context);
	}	
	
	public void saveProject(RequestContext context) {	
		logger.info("saveProject");		
		// Retrieve contact bean
		ProjectBean projectBean = (ProjectBean) context.getFlowScope().get("projectBean");
		Project project = projectBean.getProject();		
		projectService.save(project);				
		logger.info("Project {} {} saved successfully", project.getId(), project.getName());
		//logger.info("<<<<<<<< project.getUser().getName(): {}", project.getUser().getName());
		context.getMessageContext().addMessage(new MessageBuilder().info().code("message_project_save_success").build());		
	}	

	//
	// PRIVATE
	//
	
	private Project getProject(RequestContext context) {
		logger.info("getProject");		
		Long id = context.getRequestScope().getLong("projectId");		
		logger.info("Selected project id: {}", id);		
		return projectService.findById(id);			
	}
	
	private List<User> getUsers() {
		logger.info("getUsers");		
		List<User> users = this.userService.findAll();         
        return users;
	}
}
