package cat.xaviermarin.pm.web.controller;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.binding.message.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.webflow.execution.RequestContext;

import cat.xaviermarin.pm.domain.Milestone;
import cat.xaviermarin.pm.service.MilestoneService;
import cat.xaviermarin.pm.web.view.MilestoneBean;
import cat.xaviermarin.pm.web.view.MilestoneListBean;

@Component("milestoneController")
public class MilestoneController extends BaseController {

	private static final Logger logger = LoggerFactory.getLogger(MilestoneController.class);

	@Autowired
	private MilestoneService milestoneService;
	
	//
	// BEANS MANAGEMENT
	//
	
	// RECOVERS MILESTONES LIST AND SET IT IN LISTBEAN
	public MilestoneListBean getMilestoneListBean() {		
		logger.info("MilestoneController.getMilestoneListBean");
		MilestoneListBean milestoneListBean = new MilestoneListBean();		
		milestoneListBean.setMilestones(milestoneService.findAll());		
		return milestoneListBean;		
	}	
	public MilestoneListBean getMilestoneListBean(Long projectId) {
		logger.info("MilestoneController.getMilestoneListBean with projectId: {}", projectId);		
		MilestoneListBean milestoneListBean = new MilestoneListBean();		
		milestoneListBean.setMilestones(milestoneService.findAllByProjectId(projectId));		
		return milestoneListBean;	
	}
	
	// CREATES A NEW EMPTY MILESTONE BEAN
	public MilestoneBean newMilestoneBean() {		
		logger.info("newMilestoneBean");		
		
		MilestoneBean milestoneBean = new MilestoneBean();		
		Milestone milestone = new Milestone();
		milestoneBean.setMilestone(milestone);			       
		
        milestoneBean.setMilestones(this.getMilestones());		
		return milestoneBean;		
	}

	// CREATES A FULL MILESTONE BEAN
	public MilestoneBean getMilestoneBean(RequestContext context) {		
		logger.info("getMilestoneBean");
		
		MilestoneBean milestoneBean = new MilestoneBean();	
		Milestone milestone = new Milestone();		
		milestone = this.getMilestone(context);	
		milestoneBean.setMilestone(milestone);	
		
		milestoneBean.setMilestones(this.getMilestones());
		
		return milestoneBean;		
	}
	
	//
	// VIEW ACTIONS
	//
	
	public void deleteMilestone(RequestContext context) {
		logger.info("deleteMilestone");
		Long id = context.getRequestScope().getLong("milestoneId");
		logger.info("Selected milestone id: {}", id);
		milestoneService.delete(id);
	}
	
	public Milestone showMilestone(RequestContext context) {		
		logger.info("showMilestone");		
		return this.getMilestone(context);
	}	
	
	public void saveMilestone(RequestContext context) {	
		logger.info("saveMilestone");		
		// Retrieve contact bean
		MilestoneBean milestoneBean = (MilestoneBean) context.getFlowScope().get("milestoneBean");
		Milestone milestone = milestoneBean.getMilestone();		
		milestoneService.save(milestone);				
		logger.info("Milestone {} {} saved successfully", milestone.getId(), milestone.getName());
		//logger.info("<<<<<<<< milestone.getMilestone().getName(): {}", milestone.getMilestone().getName());
		context.getMessageContext().addMessage(new MessageBuilder().info().code("message_milestone_save_success").build());		
	}	

	//
	// PRIVATE
	//
	
	private Milestone getMilestone(RequestContext context) {
		logger.info("getMilestone");		
		Long id = context.getRequestScope().getLong("milestoneId");		
		logger.info("Selected milestone id: {}", id);		
		return milestoneService.findById(id);			
	}
	
	private List<Milestone> getMilestones() {
		logger.info("getMilestones");		
		List<Milestone> milestones = this.milestoneService.findAll();         
        return milestones;
	}
}
