package cat.xaviermarin.pm.web.controller;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.binding.message.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.webflow.execution.RequestContext;

import cat.xaviermarin.pm.domain.Task;
import cat.xaviermarin.pm.service.TaskService;
import cat.xaviermarin.pm.web.view.TaskBean;
import cat.xaviermarin.pm.web.view.TaskListBean;

@Component("taskController")
public class TaskController extends BaseController {

	private static final Logger logger = LoggerFactory.getLogger(TaskController.class);

	@Autowired
	private TaskService taskService;
	
	//
	// BEANS MANAGEMENT
	//
	
	// RECOVERS MILESTONES LIST AND SET IT IN LISTBEAN
	public TaskListBean getTaskListBean() {		
		logger.info("TaskController.getTaskListBean");
		TaskListBean taskListBean = new TaskListBean();		
		taskListBean.setTasks(taskService.findAll());		
		return taskListBean;		
	}
	public TaskListBean getTaskListBean(Long projectId) {
		logger.info("TaskController.getTaskListBean with projectId: {}", projectId);		
		TaskListBean taskListBean = new TaskListBean();		
		taskListBean.setTasks(taskService.findAllByProjectId(projectId));		
		return taskListBean;	
	}
	
	// CREATES A NEW EMPTY MILESTONE BEAN
	public TaskBean newTaskBean() {		
		logger.info("newTaskBean");		
		
		TaskBean taskBean = new TaskBean();		
		Task task = new Task();
		taskBean.setTask(task);			       
		
        taskBean.setTasks(this.getTasks());		
		return taskBean;		
	}

	// CREATES A FULL MILESTONE BEAN
	public TaskBean getTaskBean(RequestContext context) {		
		logger.info("getTaskBean");
		
		TaskBean taskBean = new TaskBean();	
		Task task = new Task();		
		task = this.getTask(context);	
		taskBean.setTask(task);	
		
		taskBean.setTasks(this.getTasks());
		
		return taskBean;		
	}
	
	//
	// VIEW ACTIONS
	//
	
	public void deleteTask(RequestContext context) {
		logger.info("deleteTask");
		Long id = context.getRequestScope().getLong("taskId");
		logger.info("Selected task id: {}", id);
		taskService.delete(id);
	}
	
	public Task showTask(RequestContext context) {		
		logger.info("showTask");		
		return this.getTask(context);
	}	
	
	public void saveTask(RequestContext context) {	
		logger.info("saveTask");		
		// Retrieve contact bean
		TaskBean taskBean = (TaskBean) context.getFlowScope().get("taskBean");
		Task task = taskBean.getTask();		
		taskService.save(task);				
		logger.info("Task {} {} saved successfully", task.getId(), task.getName());
		//logger.info("<<<<<<<< task.getTask().getName(): {}", task.getTask().getName());
		context.getMessageContext().addMessage(new MessageBuilder().info().code("message_task_save_success").build());		
	}	

	//
	// PRIVATE
	//
	
	private Task getTask(RequestContext context) {
		logger.info("getTask");		
		Long id = context.getRequestScope().getLong("taskId");		
		logger.info("Selected task id: {}", id);		
		return taskService.findById(id);			
	}
	
	private List<Task> getTasks() {
		logger.info("getTasks");		
		List<Task> tasks = this.taskService.findAll();         
        return tasks;
	}
}
