package cat.xaviermarin.pm.web.controller;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.binding.message.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.webflow.execution.RequestContext;

import cat.xaviermarin.pm.domain.User;
import cat.xaviermarin.pm.domain.Role;
import cat.xaviermarin.pm.service.UserService;
import cat.xaviermarin.pm.service.RoleService;
import cat.xaviermarin.pm.web.view.UserBean;
import cat.xaviermarin.pm.web.view.UserListBean;

@Component("userController")
public class UserController extends BaseController {

	private static final Logger logger = LoggerFactory.getLogger(UserController.class);

	@Autowired
	private UserService userService;
	
	@Autowired
	private RoleService roleService;
	
	//
	// BEANS MANAGEMENT
	//
	
	// RECOVERS ROLES LIST AND SET IT IN LISTBEAN
	public UserListBean getUserListBean() {		
		logger.info("UserController.getUserListBean");
		UserListBean userListBean = new UserListBean();		
		userListBean.setUsers(userService.findAll());		
		return userListBean;		
	}
	
	// CREATES A NEW EMPTY ROLE BEAN
	public UserBean newUserBean() {		
		logger.info("newUserBean");		
		
		UserBean userBean = new UserBean();		
		User user = new User();
		userBean.setUser(user);			       
		
        userBean.setRoles(this.getRoles());		
		return userBean;		
	}

	// CREATES A FULL ROLE BEAN
	public UserBean getUserBean(RequestContext context) {		
		logger.info("getUserBean");
		
		UserBean userBean = new UserBean();	
		User user = new User();		
		user = this.getUser(context);	
		userBean.setUser(user);	
		
		userBean.setRoles(this.getRoles());
		
		return userBean;		
	}
	
	//
	// VIEW ACTIONS
	//
	
	public void deleteUser(RequestContext context) {
		logger.info("deleteUser");
		Long id = context.getRequestScope().getLong("userId");
		logger.info("Selected user id: {}", id);
		userService.delete(id);
	}
	
	public User showUser(RequestContext context) {		
		logger.info("showUser");		
		return this.getUser(context);
	}	
	
	public void saveUser(RequestContext context) {	
		logger.info("saveUser");		
		// Retrieve contact bean
		UserBean userBean = (UserBean) context.getFlowScope().get("userBean");
		User user = userBean.getUser();		
		userService.save(user);				
		logger.info("User {} {} saved successfully", user.getId(), user.getLogin());
		//logger.info("<<<<<<<< user.getRole().getName(): {}", user.getRole().getName());
		context.getMessageContext().addMessage(new MessageBuilder().info().code("message_user_save_success").build());		
	}	

	//
	// PRIVATE
	//
	
	private User getUser(RequestContext context) {
		logger.info("getUser");		
		Long id = context.getRequestScope().getLong("userId");		
		logger.info("Selected user id: {}", id);		
		return userService.findById(id);			
	}
	
	private List<Role> getRoles() {
		logger.info("getRoles");		
		List<Role> roles = this.roleService.findAll();         
        return roles;
	}
}
