DROP TABLE users IF EXISTS;
DROP TABLE roles IF EXISTS;
DROP TABLE projects IF EXISTS;
DROP TABLE milestones IF EXISTS;

CREATE TABLE roles (
	id			INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) PRIMARY KEY,
	name		VARCHAR(50)
);

CREATE TABLE users (
	id				INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) PRIMARY KEY,
	login			VARCHAR(50),
	firstName		VARCHAR(50),
	lastName		VARCHAR(50),
	email			VARCHAR(50),
	password		VARCHAR(50),
	enabled			BOOLEAN,
	role_id			INTEGER
);
ALTER TABLE users ADD CONSTRAINT fk_users_roles FOREIGN KEY (role_id) REFERENCES roles (id);

CREATE TABLE projects (
	id				INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) PRIMARY KEY,
	name			VARCHAR(50),
	description		VARCHAR(255),
	webAddress		VARCHAR(100),
	director_id		INTEGER,
	chief_id		INTEGER
);
ALTER TABLE projects ADD CONSTRAINT fk_projects_directors FOREIGN KEY (director_id) REFERENCES users (id);
ALTER TABLE projects ADD CONSTRAINT fk_projects_chiefs FOREIGN KEY (chief_id) REFERENCES users (id);

CREATE TABLE milestones (
	id				INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) PRIMARY KEY,
	name			VARCHAR(50),
	description		VARCHAR(255),
	project_id		INTEGER,
	startDate		VARCHAR(10),
	endDate			VARCHAR(10)
);
ALTER TABLE milestones ADD CONSTRAINT fk_milestones_projects FOREIGN KEY (project_id) REFERENCES projects (id);

CREATE TABLE tasks (
	id				INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) PRIMARY KEY,
	name			VARCHAR(50),
	description		VARCHAR(255),
	project_id		INTEGER,
	milestone_id	INTEGER,
	startDate		VARCHAR(10),
	endDate			VARCHAR(10)
);
ALTER TABLE tasks ADD CONSTRAINT fk_tasks_projects FOREIGN KEY (project_id) REFERENCES projects (id);
ALTER TABLE tasks ADD CONSTRAINT fk_tasks_milestones FOREIGN KEY (milestone_id) REFERENCES milestones (id);

CREATE TABLE assignations (
	id				INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) PRIMARY KEY,
	description		VARCHAR(255),
	spendTime		INTEGER,
	task_id			INTEGER
);
ALTER TABLE assignations ADD CONSTRAINT fk_assignations_tasks FOREIGN KEY (task_id) REFERENCES tasks (id);
